/*
 * AD7321_mango_porting.c
 */

#include "AD7321_mango_porting.h"
#include "misc.h"
#include "stm32f10x_exti.h"
#include "stm32f10x_gpio.h"

#define AD7321_SPI_BEGIN()  GPIO_ResetBits(GPIO_AD7321_SPI, GPIO_AD7321_SPI_AD_CS_PIN)
#define AD7321_SPI_END()    GPIO_SetBits(GPIO_AD7321_SPI, GPIO_AD7321_SPI_AD_CS_PIN)

// SPI access macros

static void AD7321_SPI_TX(u16 data)
{
    /* Wait for SPI1 Tx buffer empty */
    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);
    /* Send SPI1 data */
    SPI_I2S_SendData(SPI1, data);
}

static u16 AD7321_SPI_RX(void)
{
    return SPI_I2S_ReceiveData(SPI1);
}

static void AD7321_SPI_WAIT_RXRDY(void)
{
    /* Wait for SPI1 data reception */
    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
}

static u16 AD7321_SPI_TXRX(u16 x)
{
    AD7321_SPI_TX(x);
    AD7321_SPI_WAIT_RXRDY();
    return AD7321_SPI_RX();
}

u16 AD7321_SPI_Read_16(void)
{
    u16 data;

    AD7321_SPI_BEGIN();

    data = AD7321_SPI_TXRX(0x0);

    AD7321_SPI_END();

    AD7321_SPI_PRINTK("Read RegData: 0x%0X\n", data);
    return data;
}

void AD7321_SPI_Write_16(u16 data)
{
    AD7321_SPI_BEGIN();
    
    AD7321_SPI_TXRX(data);

    AD7321_SPI_END();

    AD7321_SPI_PRINTK("Write RegData: 0x%0X\n", data);
}

