/*
 * AD7321 ADC driver
 */

#include "ad7321.h"

//#define AD7321_USE_SOFTWARE_SPI

#ifdef AD7321_USE_SOFTWARE_SPI
#include "ad7321_spi_sw.h"
#else
#include "AD7321_mango_porting.h"
#endif

static long ad7321_device_ioctl(unsigned int, int *);
static int ad7321_spi_read(u16 * val);
static int ad7321_spi_write(u16 val);

/*
 * Device dependant operations
 */

#define AD7321_ADC_READ_COUNT 32

static long ad7321_device_ioctl(unsigned int cmd, int * arg_p)
{
    int i;
    u16 value;
    s32 s_value, s_value_sum = 0;

    AD7321PRINTK("+++\n");

    switch(cmd)
    {
    case AD7321_IOCTL_SET_CHANNEL:
        AD7321PRINTK("AD7321_IOCTL_SET_CHANNEL: %d\n", * arg_p);
        if(0 == * arg_p)
        {
            value = AD7321_WRITE_BIT | AD7321_REG_SELECT_CONTROL
                    | AD7321_CTRL_CHANNEL_0 | AD7321_CTRL_REF_CODING_STRAIGHT_BINARY | AD7321_CTRL_REF_ENABLE;
        }
        else if(1 == * arg_p)
        {
            value = AD7321_WRITE_BIT | AD7321_REG_SELECT_CONTROL
                    | AD7321_CTRL_CHANNEL_1 | AD7321_CTRL_REF_CODING_STRAIGHT_BINARY | AD7321_CTRL_REF_ENABLE;
        }
        else
        {
            AD7321PRINTK("[ERROR] Channel ID Fail: %d\n", * arg_p);
            return -1;
        }
        ad7321_spi_write(value);
        AD7321PRINTK("Write Range Reg: 0x%02X\n", value);
        break;

    case AD7321_IOCTL_GET_ADC_DATA:
        AD7321PRINTK("AD7321_IOCTL_GET_ADC_DATA\n");

        for(i = 0; i < AD7321_ADC_READ_COUNT; i ++)
        {
            ad7321_spi_read(&value);
            AD7321PRINTK("org RegVal %d: 0x%02X, %d\n", i, value, value);

            value = value & 0x1FFF;

            if(value & 0x1000)
            {
                s_value = value | 0xFFFFF000;
                AD7321PRINTK("Minux RegVal %d: %d\n", i, s_value);
            }
            else
            {
                s_value = value;
                AD7321PRINTK("Plus RegVal %d: %d\n", i, s_value);
            }
            s_value_sum += s_value;
            Delay(1);
        }
        * arg_p = (int)(s_value_sum / AD7321_ADC_READ_COUNT);
        AD7321PRINTK("s_value_sum: %d, * arg_p: %d\n", s_value_sum, * arg_p);
        break;

    default:
        return -1;
    }

    return 0;
}

/*
 * device operation
 */

static int ad7321_spi_read(u16 * val)
{
#ifdef AD7321_USE_SOFTWARE_SPI
    * val = SW_SPI_AD7321_Read_16();
#else
    * val = AD7321_SPI_Read_16();
#endif
    return 0;
}

static int ad7321_spi_write(u16 val)
{
#ifdef AD7321_USE_SOFTWARE_SPI
    SW_SPI_AD7321_Write_16(val);
#else
    AD7321_SPI_Write_16(val);
#endif
    return 0;
}

void AD7321_HW_Init(void)
{
    u16 value;
    
    AD7321PRINTK("+++\n");

#ifdef AD7321_USE_SOFTWARE_SPI
    SW_SPI_Config_Start();
#endif

    value = AD7321_WRITE_BIT | AD7321_REG_SELECT_RANGE
            | AD7321_RANGE_CH0_0_TO_10 | AD7321_RANGE_CH1_0_TO_10;
    ad7321_spi_write(value);

    AD7321PRINTK("Write Range Reg: 0x%02X\n", value);
}

int Set_AD7321_ADC_Channel(int channel)
{
//    printf("[CRZ] %s - channel: %d\n", __func__, channel);
    if(ad7321_device_ioctl(AD7321_IOCTL_SET_CHANNEL, &channel) < 0)
    {
        printf("[ERROR] AD7321_IOCTL_SET_CHANNEL IOCTL error\n");
        return -1;
    }

    return 0;
}

int Get_AD7321_ADC_Data(int channel)
{
    int value;
//    printf("[CRZ] %s\n", __func__);

    Set_AD7321_ADC_Channel(channel);

    if(ad7321_device_ioctl(AD7321_IOCTL_GET_ADC_DATA, &value) < 0)
    {
        printf("[ERROR] AD7321_IOCTL_GET_ADC_DATA IOCTL error\n");
        return -1;
    }

    return value;
}

