/*
 * (C) COPYRIGHT 2009 CRZ
 *
 * File Name : spi_sw.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 09/23/2009
 */

#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/i2c.h>
#include <linux/init.h>
#include <linux/input.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/mutex.h>
#include <linux/types.h>
#include <linux/workqueue.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>

#include <mach/gpio.h>
#include <plat/gpio-cfg.h>

#include "ad7321_spi_sw.h"

#define SW_SPI_WAIT_TIME    50
#define SW_SPI_WAIT_TIME_2  10

typedef enum
{
    Bit_RESET = 0,
    Bit_SET
} BitAction;

unsigned int SW_SPI_CPHA_val = 0;
unsigned int SW_SPI_CPOL_val = 1;

#define GPIO_SW_SPI_MOSI_PIN  S3C2410_GPE(12)
#define GPIO_SW_SPI_MISO_PIN  S3C2410_GPE(11)
#define GPIO_SW_SPI_CLK_PIN   S3C2410_GPE(13)
#define GPIO_SW_SPI_NCS_PIN   S3C2410_GPL(13)

#define SPI_BEGIN() \
{ \
    SW_SPI_CLK_Init(); \
    TIMER_Wait_us(SW_SPI_WAIT_TIME_2); \
    gpio_set_value(GPIO_SW_SPI_NCS_PIN, 0); \
}

#define SPI_END()    gpio_set_value(GPIO_SW_SPI_NCS_PIN, 1)

void SW_SPI_CLK_Init(void);
unsigned char SW_SPI_TXRX(unsigned char tx_data);
u16 SW_SPI_TXRX_16(u16 tx_data);

void TIMER_Wait_us(volatile unsigned int nCount)
{
    for (; nCount != 0;nCount--);
}

void TIMER_Wait_ms(volatile unsigned int nCount)
{
    for (; nCount != 0; nCount--)
        TIMER_Wait_us(56000);
}

void SW_SPI_Config_Start(void)
{
    AD7321_SPI_SW_PRINTK("+++\n");

    s3c_gpio_cfgpin(GPIO_SW_SPI_MISO_PIN, S3C_GPIO_INPUT); // INPUT mode
	s3c_gpio_setpull(GPIO_SW_SPI_MISO_PIN, S3C_GPIO_PULL_NONE);

    s3c_gpio_cfgpin(GPIO_SW_SPI_MOSI_PIN, S3C_GPIO_OUTPUT); // OUTPUT mode
	s3c_gpio_setpull(GPIO_SW_SPI_MOSI_PIN, S3C_GPIO_PULL_NONE);

	s3c_gpio_cfgpin(GPIO_SW_SPI_CLK_PIN, S3C_GPIO_OUTPUT); // OUTPUT mode
	s3c_gpio_setpull(GPIO_SW_SPI_CLK_PIN, S3C_GPIO_PULL_NONE);

	s3c_gpio_cfgpin(GPIO_SW_SPI_NCS_PIN, S3C_GPIO_OUTPUT); // OUTPUT mode
	s3c_gpio_setpull(GPIO_SW_SPI_NCS_PIN, S3C_GPIO_PULL_NONE);

    SW_SPI_CLK_Init();
}

void SW_SPI_Config_End(void)
{
    AD7321_SPI_SW_PRINTK("+++\n");
}

unsigned char SW_SPI_Read_Register(unsigned char addr)
{
    unsigned char data;

    SPI_BEGIN();

    // 0x80 is Read/Write Bit
    addr = 0x80 | addr;
    
    SW_SPI_TXRX(addr);
    data = SW_SPI_TXRX(0x0);

    SPI_END();

    AD7321_SPI_SW_PRINTK("RegAddr: 0x%0X, RegData: 0x%0X\n", addr, data);
    return data;
}

void SW_SPI_Write_Register(unsigned char addr, unsigned char data)
{
    SPI_BEGIN();
    
    SW_SPI_TXRX(addr);
    SW_SPI_TXRX(data);

    SPI_END();

    AD7321_SPI_SW_PRINTK("RegAddr: 0x%0X, RegData: 0x%0X\n", addr, data);
}

u16 SW_SPI_AD7321_Read_16(void)
{
    u16 data;

    SPI_BEGIN();

    data = SW_SPI_TXRX_16(0x0);

    SPI_END();

    AD7321_SPI_SW_PRINTK("RegData: 0x%0X\n", data);
    return data;
}

void SW_SPI_AD7321_Write_16(u16 data)
{
    SPI_BEGIN();
    
    SW_SPI_TXRX_16(data);

    SPI_END();

    AD7321_SPI_SW_PRINTK("RegData: 0x%0X\n", data);
}

unsigned char SW_SPI_ReadVal_MISO(void)
{
    return gpio_get_value(GPIO_SW_SPI_MISO_PIN);
}

// SPI MOSI High
void SW_SPI_MOSI_H(void)
{
	gpio_set_value(GPIO_SW_SPI_MOSI_PIN, 1);
}

// SPI MOSI low
void SW_SPI_MOSI_L(void)
{
	gpio_set_value(GPIO_SW_SPI_MOSI_PIN, 0);
}

// SPI CLK High
void SW_SPI_CLK_H(void)
{
	gpio_set_value(GPIO_SW_SPI_CLK_PIN, 1);
}

// SPI CLK low
void SW_SPI_CLK_L(void)
{
	gpio_set_value(GPIO_SW_SPI_CLK_PIN, 0);
}

void SW_SPI_MOSI_OUT(u16 out)
{
    if (out)
    {
        SW_SPI_MOSI_H();
    }
    else
    {
        SW_SPI_MOSI_L();
    }
    TIMER_Wait_us(SW_SPI_WAIT_TIME);
}

void SW_SPI_CLK_Init(void)
{
    if(0 == SW_SPI_CPOL_val)
    {
        SW_SPI_CLK_L();
    }
    else
    {
        SW_SPI_CLK_H();
    }
    TIMER_Wait_us(SW_SPI_WAIT_TIME);
}

void SW_SPI_CLK_Toggle(void)
{
    gpio_set_value(GPIO_SW_SPI_CLK_PIN,
        (BitAction)(1-gpio_get_value(GPIO_SW_SPI_CLK_PIN)));

    TIMER_Wait_us(SW_SPI_WAIT_TIME);
}

unsigned char SW_SPI_TXRX(unsigned char tx_data)
{
    unsigned char  rx_data = 0;

    unsigned int i;
    unsigned char bitCheckVal = 0x80;

    if(1 == SW_SPI_CPHA_val)
    {
        SW_SPI_CLK_Toggle();    
    }

	for (i=0; i<8; i++) 
	{
		SW_SPI_MOSI_OUT(tx_data & bitCheckVal);
		bitCheckVal >>= 1;

        SW_SPI_CLK_Toggle();    

        TIMER_Wait_us(SW_SPI_WAIT_TIME_2);

        rx_data <<= 1;
        if(SW_SPI_ReadVal_MISO() == Bit_SET)
        {
            rx_data |= 0x1;
        }

        if((7 == i) && (1 == SW_SPI_CPHA_val))
        {    // In last loop, if CPHA is 1, should skip CLK Toggle ...
            break;
        }
        SW_SPI_CLK_Toggle();    
	}

    TIMER_Wait_us(SW_SPI_WAIT_TIME);
    return rx_data;
}

u16 SW_SPI_TXRX_16(u16 tx_data)
{
    u16  rx_data = 0;

    unsigned int i;
    u16 bitCheckVal = 0x8000;

    if(1 == SW_SPI_CPHA_val)
    {
        SW_SPI_CLK_Toggle();    
    }

	for (i=0; i<16; i++) 
	{
		SW_SPI_MOSI_OUT(tx_data & bitCheckVal);
		bitCheckVal >>= 1;

        SW_SPI_CLK_Toggle();    

        rx_data <<= 1;
        if(SW_SPI_ReadVal_MISO() == Bit_SET)
        {
            rx_data |= 0x1;
        }

        if((15 == i) && (1 == SW_SPI_CPHA_val))
        {    // In last loop, if CPHA is 1, should skip CLK Toggle ...
            break;
        }
        SW_SPI_CLK_Toggle();    
	}

    TIMER_Wait_us(SW_SPI_WAIT_TIME);
    return rx_data;
}

