/*
 * cc2520_mango_porting.c
 */

#include "hw_config.h"

#include "misc.h"
#include "stm32f10x_exti.h"
#include "stm32f10x_gpio.h"

void AT86RF212_GPIO_0_Interrupt_Setting(FunctionalState  fStateE)
{
    NVIC_InitTypeDef NVIC_InitStructure;
    EXTI_InitTypeDef EXTI_InitStructure;

    /* Configure one bit for preemption priority */
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

    /* Enable the EXTI8 Interrupt */
    NVIC_InitStructure.NVIC_IRQChannel                   = g_at86rf212_gpio_pin_0_irq;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority        = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd                = fStateE;
    NVIC_Init(&NVIC_InitStructure);

    /* Connect EXTI Line to gpio pin */
    GPIO_EXTILineConfig(GPIO_PORTSOURCE_AT86RF212_GPIO0, g_at86rf212_gpio_pin_0_pin_source);

    /* Configure EXTI Line to generate an interrupt */
    EXTI_ClearITPendingBit(g_at86rf212_gpio_pin_0_exti_line);

    EXTI_InitStructure.EXTI_Line    = g_at86rf212_gpio_pin_0_exti_line;
    EXTI_InitStructure.EXTI_Mode    = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
    EXTI_InitStructure.EXTI_LineCmd = fStateE;
    EXTI_Init(&EXTI_InitStructure);
}

static void at86rf212_reset_opin(u8 data)
{
    if(0 == data)
    {
        GPIO_SetBits(GPIO_SPI_IO, g_at86rf212_gpio_pin_reset);
    }
    else
    {
        GPIO_ResetBits(GPIO_SPI_IO, g_at86rf212_gpio_pin_reset);
    }
}

void AT86RF212_RESET(void)
{
    // IO7 pin sould be Low
    at86rf212_reset_opin(0);
    Delay(100);
    at86rf212_reset_opin(1);
    Delay(100);

    // IO7 pin sould be Low
    at86rf212_reset_opin(0);
    Delay(100);
    at86rf212_reset_opin(1);
    Delay(100);
}

void AT86RF212_SPI_TX(u8 data)
{
    /* Wait for SPI Tx buffer empty */
    while (SPI_I2S_GetFlagStatus(g_at86rf212_spi_num_p, SPI_I2S_FLAG_TXE) == RESET);
    /* Send SPI data */
    SPI_I2S_SendData(g_at86rf212_spi_num_p, data);
}

u8 AT86RF212_SPI_RX(void)
{
    return SPI_I2S_ReceiveData(g_at86rf212_spi_num_p);
}

void AT86RF212_SPI_WAIT_RXRDY(void)
{
    /* Wait for SPI data reception */
    while (SPI_I2S_GetFlagStatus(g_at86rf212_spi_num_p, SPI_I2S_FLAG_RXNE) == RESET);
}

uint8 AT86RF212_SPI_TXRX(uint8 x)
{
    AT86RF212_SPI_TX(x);
    AT86RF212_SPI_WAIT_RXRDY();
    return AT86RF212_SPI_RX();
}

