#ifndef __BSHARP_COMM_H
#define __BSHARP_COMM_H

/* Includes ------------------------------------------------------------------*/

#include "stm32f10x.h"
#include "stdio.h"

/*
Description           Com  Type 1st  2nd (opcode) 3rd(Length)
ACK REQUEST           0x00 0x40 0x40 0x00         0x00
SEND TIME DATA        0x80 0x08 0x88 0x00         # of bytes
SEND VOLT DATA        0x80 0x06 0x86 0x00         # of bytes
SEND MAX_TICK         0x80 0x00 0x80 0x00         0x01
SEND PRESCALER        0x80 0x01 0x81 0x00         0x01
SEND CURRENT REQUST   0x00 0x07 0x07 0x00         0x00
SEND VOLTAGE REQUST   0x00 0x06 0x06 0x00         0x00
SEND CUR AVG. REQUST  0x00 0x05 0x05 0x00         0x00
SEND MEASURE VOLTAGE  0x80 0x** 0x80 0x80         0x00
SEND MEASURE CURRENT  0x80 0x** 0x80 0x40         0x00
SEND MEASURE CUR AVG. 0x80 0x** 0x80 0x20         0x00
SEND EMULATE VOLTAGE  0x80 0x** 0x80 0x10         0x00
*/

/* Exported types ------------------------------------------------------------*/

#define	BSHARP_COUNT_TYPE	unsigned char
#define	BSHARP_DATA_TYPE	unsigned short int

/* Exported constants --------------------------------------------------------*/

#define	BSHARP_PACKET_SIZE	    89

#define	BSHARP_PREAMBLE_SIZE        6
#define	BSHARP_PREAMBLE_CONTENT     "\x00\x00\xb4\x8a\x8b\x3f"

#define	BSHARP_COMMAND_SIZE         3
#define	BSHARP_DATA_MAX_SIZE        (BSHARP_PACKET_SIZE - BSHARP_PREAMBLE_SIZE - BSHARP_COMMAND_SIZE)

#define	BSHARP_ACK_SIZE	        (BSHARP_PREAMBLE_SIZE + 2)

#define	BSHARP_TYPECODE_OFFSET  BSHARP_PREAMBLE_SIZE
#define	BSHARP_OPCODE_OFFSET	(BSHARP_PREAMBLE_SIZE + 1)
#define	BSHARP_COUNT_OFFSET	    (BSHARP_PREAMBLE_SIZE + 2)
#define	BSHARP_DATA_OFFSET	    (BSHARP_COUNT_OFFSET + sizeof(BSHARP_COUNT_TYPE))

#define	BSHARP_DATA_TYPE_MASK	'\x0f'
#define	BSHARP_MAX_TICK	        '\x00'
#define	BSHARP_PRESCALAR	    '\x01'
#define	BSHARP_ENDIAN	        '\x02'  /* 0 : for PC, 1 for MAC */
#define	BSHARP_DEBUG_FLAG1	    '\x03'
#define	BSHARP_DEBUG_FLAG2	    '\x04'

//#define BSHARP_VOLTAGE_REF0     '\x03'
//#define BSHARP_VOLTAGE_REF_HI   '\x04'
//#define BSHARP_VOLTAGE_REF_LO   '\x05'
#define	BSHARP_VOLTAGE          '\x06'
#define	BSHARP_AVG_CURRENT      '\x05'
#define	BSHARP_CURRENT          '\x07'
#define	BSHARP_TIME             '\x08'

#define BSHARP_VREFADC_INDEX_MASK   0x09
#define BSHARP_VREF_5               0x00
#define BSHARP_VREF_4               0x01

#define	BSHARP_COMM_TYPE_MASK       '\xf0'
#define	BSHARP_SEND                 '\x80'
#define	BSHARP_RECV                 '\x00'
#define	BSHARP_SEND_RECV_MASK       '\x80'
#define	BSHARP_ACK                  '\x40'
#define	BSHARP_NOACK                '\x00'
#define	BSHARP_ACK_NOACK_MASK       '\x40'
#define	BSHARP_STOP                 '\x00'
#define	BSHARP_CONTINUE             '\x20'
#define	BSHARP_STOP_CONTINUE_MASK   '\x20'

#define	BSHARP_MEASURE_VOLTAGE      '\x80'
#define	BSHARP_MEASURE_CURRENT      '\x40'
#define	BSHARP_MEASURE_AVG_CURRENT  '\x20'
#define	BSHARP_EMULATE_VOLTAGE      '\x10'
#define	BSHARP_NOP                  '\x00'

#define	BSHARP_ITEM_COUNT_PER_PACKET ((PACKET_SIZE - BSHARP_DATA_OFFSET) / sizeof(BSHARP_DATA_TYPE))

/* Exported macro ------------------------------------------------------------*/

/* External variables --------------------------------------------------------*/

/* Exported functions ------------------------------------------------------- */

void BSharp_Packet_Print(unsigned char * data_p, unsigned int size);
int BSharp_Packet_Check_Preamble(unsigned char * data_p);
void BSharp_Packet_Get_Data(unsigned char * data_p, unsigned int size);

#endif  /* __BSHARP_COMM_H */

