/*
 * (C) COPYRIGHT 2009 CRZ
 *
 * File Name : hw_config.h
 * Author    : POOH
 * Version   : V1.0
 * Date      : 08/12/2009
 */

#ifndef __HW_CONFIG_H
#define __HW_CONFIG_H

typedef enum
{
  FALSE = 0, TRUE  = !FALSE
}
bool;

typedef enum
{
    TEST_MODE_CC2520_TEST    = 1,
    TEST_MODE_CC1120_TEST    = 2,
    TEST_MODE_ENC28J60_TEST  = 3,
    TEST_MODE_AT86RF212_TEST = 4
} TestMode_E;

/* includes */

#include "stm32f10x.h"

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include "debug.h"
#include "led.h"
#include "key.h"

#include "i2c_driver_old.h"
#include "I2CRoutines.h"
#include "clcd_lmc1623.h"

#include "uart_queue.h"
#include "zigbee_test.h"
#include "cc1120_test.h"
#include "enc28j60_test.h"

#include "AT86RF212_mango_porting.h"
#include "AT86RF212_test.h"
#include "halTimer.h"
#include "config.h"

/* global variables */

extern bool g_TestProcessState;

extern TestMode_E  g_test_mode;

/* defines */

#define GPIO_LED             GPIOC
#define GPIO_KEY             GPIOC

#define GPIO_LCD_CTRL        GPIOE
#define GPIO_LCD_DATA        GPIOE

#define GPIO_USART           GPIOA

#define GPIO_SPI_1           GPIOA
#define GPIO_SPI_2           GPIOB
#define GPIO_SPI_IO          GPIOD

#define GPIO_LED1_PIN        GPIO_Pin_8   /* PC8 */
#define GPIO_LED2_PIN        GPIO_Pin_9   /* PC9 */
#define GPIO_LED3_PIN        GPIO_Pin_10  /* PC10 */

#define GPIO_KEY1_PIN        GPIO_Pin_11  /* PC11 */
#define GPIO_KEY2_PIN        GPIO_Pin_12  /* PC12 */
#define GPIO_KEY3_PIN        GPIO_Pin_13  /* PC13 */

#define GPIO_LCD_RS_PIN      GPIO_Pin_0   /* PE0 */
#define GPIO_LCD_NRW_PIN     GPIO_Pin_1   /* PE1 */
#define GPIO_LCD_ENABLE_PIN  GPIO_Pin_2   /* PE2 */

#define GPIO_LCD_DATA0_PIN   GPIO_Pin_3   /* PE3 */
#define GPIO_LCD_DATA1_PIN   GPIO_Pin_4   /* PE4 */
#define GPIO_LCD_DATA2_PIN   GPIO_Pin_5   /* PE5 */
#define GPIO_LCD_DATA3_PIN   GPIO_Pin_6   /* PE6 */
#define GPIO_LCD_DATA4_PIN   GPIO_Pin_7   /* PE7 */
#define GPIO_LCD_DATA5_PIN   GPIO_Pin_8   /* PE8 */
#define GPIO_LCD_DATA6_PIN   GPIO_Pin_9   /* PE9 */
#define GPIO_LCD_DATA7_PIN   GPIO_Pin_10  /* PE10 */

#define GPIO_USART_1_Rx_Pin    GPIO_Pin_10  /* PA10 */
#define GPIO_USART_1_Tx_Pin    GPIO_Pin_9   /* PA9 */
#define GPIO_USART_2_Rx_Pin    GPIO_Pin_3   /* PA3 */
#define GPIO_USART_2_Tx_Pin    GPIO_Pin_2   /* PA2 */

#define GPIO_SPI_1_NCS_PIN        GPIO_Pin_4 /* PA4 */
#define GPIO_SPI_1_CLK_PIN        GPIO_Pin_5 /* PA5 */
#define GPIO_SPI_1_MISO_PIN       GPIO_Pin_6 /* PA6 */
#define GPIO_SPI_1_MOSI_PIN       GPIO_Pin_7 /* PA7 */

#define GPIO_SPI_2_NCS_PIN        GPIO_Pin_12 /* PB12 */
#define GPIO_SPI_2_CLK_PIN        GPIO_Pin_13 /* PB13 */
#define GPIO_SPI_2_MISO_PIN       GPIO_Pin_14 /* PB14 */
#define GPIO_SPI_2_MOSI_PIN       GPIO_Pin_15 /* PB15 */

#define GPIO_SPI_IO_PIN_0         GPIO_Pin_0  /* PD0 */
#define GPIO_SPI_IO_PIN_1         GPIO_Pin_1  /* PD1 */
#define GPIO_SPI_IO_PIN_2         GPIO_Pin_2  /* PD2 */
#define GPIO_SPI_IO_PIN_3         GPIO_Pin_3  /* PD3 */
#define GPIO_SPI_IO_PIN_4         GPIO_Pin_4  /* PD4 */
#define GPIO_SPI_IO_PIN_5         GPIO_Pin_5  /* PD5 */
#define GPIO_SPI_IO_PIN_6         GPIO_Pin_6  /* PD6 */
#define GPIO_SPI_IO_PIN_7         GPIO_Pin_7  /* PD7 */
#define GPIO_SPI_IO_PIN_8         GPIO_Pin_8  /* PD8 */
#define GPIO_SPI_IO_PIN_9         GPIO_Pin_9  /* PD9 */
#define GPIO_SPI_IO_PIN_10        GPIO_Pin_10  /* PD10 */
#define GPIO_SPI_IO_PIN_11        GPIO_Pin_11  /* PD11 */
#define GPIO_SPI_IO_PIN_12        GPIO_Pin_12  /* PD12 */
#define GPIO_SPI_IO_PIN_13        GPIO_Pin_13  /* PD13 */
#define GPIO_SPI_IO_PIN_14        GPIO_Pin_14  /* PD14 */
#define GPIO_SPI_IO_PIN_15        GPIO_Pin_15  /* PD15 */

#define GPIO_PORTSOURCE_KEY        GPIO_PortSourceGPIOC
#define GPIO_PINSOURCE_KEY1        GPIO_PinSource11
#define GPIO_PINSOURCE_KEY2        GPIO_PinSource12
#define GPIO_PINSOURCE_KEY3        GPIO_PinSource13
#define GPIO_EXTI_Line_KEY1        EXTI_Line11
#define GPIO_EXTI_Line_KEY2        EXTI_Line12
#define GPIO_EXTI_Line_KEY3        EXTI_Line13

/* functions */

void Set_USBClock(void);
void Enter_LowPowerMode(void);
void Leave_LowPowerMode(void);
void USB_Interrupts_Config(void);
void USB_Cable_Config (FunctionalState NewState);
void RCC_Configuration(void);
void NVIC_Configuration(void);
void GPIO_Configuration(void);
void EXTI_Configuration(void);

void USART1_Init_forSerial(void);
void USART2_Init_forComm(void);

void Get_SerialNum(void);
#if 0
uint8_t USART_GetCharacter(USART_TypeDef *  usart_p);
#else
uint8_t UartSerialGetChar_Wait(void);
#endif

void LED_Test (void);
void KEY_Test (void);

void Delay(__IO uint32_t nTime);
void TimingDelay_Decrement(void);

void delay_1_second(void);
void delay_100_milli_second(void);

void SPI_Configuration_GPIO(void);

#endif  /*__HW_CONFIG_H */

