/*
 * AT86RF212_test.c
 */

/* Includes ------------------------------------------------------------------*/

#include "hw_config.h"

#include <stdbool.h>
#include "phy.h"
#include "halPhy.h"

GPIO_TypeDef *  g_at86rf212_gpio_ncs;
SPI_TypeDef *   g_at86rf212_spi_num_p;

uint16_t  g_at86rf212_gpio_pin_reset, g_at86rf212_gpio_pin_ncs;
uint16_t  g_at86rf212_gpio_pin_0, g_at86rf212_gpio_pin_1,
    g_at86rf212_gpio_pin_2, g_at86rf212_gpio_pin_3, g_at86rf212_gpio_pin_4, g_at86rf212_gpio_pin_5;

uint8_t   g_at86rf212_gpio_pin_0_pin_source, g_at86rf212_gpio_pin_0_irq;
uint32_t  g_at86rf212_gpio_pin_0_exti_line = EXTI_Line0;

static void SPI_Configuration(void)
{
    SPI_InitTypeDef  SPI_InitStructure;

    /* SPI Config */
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_Mode      = SPI_Mode_Master;
    SPI_InitStructure.SPI_DataSize  = SPI_DataSize_8b;

    SPI_InitStructure.SPI_CPOL    = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA    = SPI_CPHA_1Edge;

    SPI_InitStructure.SPI_NSS               = SPI_NSS_Soft;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_16;
    SPI_InitStructure.SPI_FirstBit          = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial     = 7;

    /* Enable SPI1 */
    SPI_Init(SPI1, &SPI_InitStructure);
    SPI_Cmd(SPI1, ENABLE);

    /* Enable SPI2 */
    SPI_Init(SPI2, &SPI_InitStructure);
    SPI_Cmd(SPI2, ENABLE);
}

void test_CW_signal_TX_Start(void)
{
    uint8_t value, value_write;

    /*
     * Step 1. Reset AT86RF212
     */
    AT86RF212_RESET();
    printf("Step 1> Reset done\n");
    Delay(100);

    /*
     * Step 2. Register access 0x0E W 0x01 Set IRQ mask register, enable IRQ_0 (PLL_LOCK)
     */
    printf("Step 2> Set IRQ mask register\n");
    value = phyReadRegister(IRQ_STATUS_REG);
    printf("IRQ_STATUS_REG: 0x%02X\n", value);
    value = phyReadRegister(IRQ_MASK_REG);
    printf("before IRQ_MASK_REG: 0x%02X\n", value);
    phyWriteRegister(IRQ_MASK_REG, PLL_LOCK_MASK);
    value = phyReadRegister(IRQ_MASK_REG);
    printf("after IRQ_MASK_REG: 0x%02X\n", value);
    Delay(100);

#if 0
    value = phyReadRegister(IRQ_STATUS_REG);
    printf("IRQ_STATUS_REG: 0x%02X\n", value);
    Delay(1000);
    value = phyReadRegister(IRQ_STATUS_REG);
    printf("IRQ_STATUS_REG: 0x%02X\n", value);
    Delay(1000);
    value = phyReadRegister(IRQ_STATUS_REG);
    printf("IRQ_STATUS_REG: 0x%02X\n", value);
    Delay(1000);
    value = phyReadRegister(IRQ_STATUS_REG);
    printf("IRQ_STATUS_REG: 0x%02X\n", value);
    Delay(1000);
    value = phyReadRegister(IRQ_STATUS_REG);
    printf("IRQ_STATUS_REG: 0x%02X\n", value);
#endif

    /*
     * Step 3. Register access 0x02 W 0x03 Set radio transceiver state TRX_OFF
     */
    printf("Step 3> Set radio transceiver state TRX_OFF\n");
    value = phyReadRegister(TRX_STATUS_REG);
    printf("TRX_STATUS_REG: 0x%02X\n", value);
    value = phyReadRegister(TRX_STATE_REG);
    printf("before TRX_STATE_REG: 0x%02X\n", value);
    phyWriteRegister(TRX_STATE_REG, TRX_CMD_FORCE_TRX_OFF);
    value = phyReadRegister(TRX_STATE_REG);
    printf("after TRX_STATE_REG: 0x%02X\n", value);

#if 0
    value = phyReadRegister(TRX_STATUS_REG);
    printf("TRX_STATUS_REG: 0x%02X\n", value);
    Delay(1000);
    value = phyReadRegister(TRX_STATUS_REG);
    printf("TRX_STATUS_REG: 0x%02X\n", value);
    Delay(1000);
    value = phyReadRegister(TRX_STATUS_REG);
    printf("TRX_STATUS_REG: 0x%02X\n", value);
    Delay(1000);
    value = phyReadRegister(TRX_STATUS_REG);
    printf("TRX_STATUS_REG: 0x%02X\n", value);
    Delay(1000);
    value = phyReadRegister(TRX_STATUS_REG);
    printf("TRX_STATUS_REG: 0x%02X\n", value);
#endif

    /*
     * Step 4. Register access W Set channel, refer to section 7.8.2.
     */
    {
        uint8_t cc_band;
        uint8_t cc_num_channel;
        float   frequency_val;

        /*
        CC_BAND CC_NUMBER Description
        1 0 . 255 769.0 . 794.5 MHz; Fc [MHz] = 769.0 + 0.1  CC_NUMBER
        2 0 . 255 857.0 . 882.5 MHz; Fc [MHz] = 857.0 + 0.1  CC_NUMBER
        3 0 . 255 903.0 . 928.5 MHz; Fc [MHz] = 903.0 + 0.1  CC_NUMBER
        4 0 . 94 769 . 863 MHz; Fc [MHz] = 769 + CC_NUMBER
        5 0 . 102 833 . 935 MHz; Fc [MHz] = 833 + CC_NUMBER
        */

        /* 914 MHz */
        cc_band = 3;
        cc_num_channel = 110;
        frequency_val = 903.0 + 0.1 * cc_num_channel;

        printf("Step 4> Set channel frequency, cc_band: %d, cc_num_channel: %d, freq: %f\n",
               cc_band, cc_num_channel, frequency_val);

        value = phyReadRegister(CC_CTRL_1_REG);
        printf("before CC_CTRL_1_REG: 0x%02X\n", value);
        value = phyReadRegister(CC_CTRL_0_REG);
        printf("before CC_CTRL_0_REG: 0x%02X\n", value);

        phyWriteRegister(CC_CTRL_1_REG, cc_band);
        phyWriteRegister(CC_CTRL_0_REG, cc_num_channel);

        value = phyReadRegister(CC_CTRL_1_REG);
        printf("after CC_CTRL_1_REG: 0x%02X\n", value);
        value = phyReadRegister(CC_CTRL_0_REG);
        printf("after CC_CTRL_0_REG: 0x%02X\n", value);
    }

    /*
     * Step 5. Register access W Set TX output power, refer to section 7.3.4.
     *     For CW mode, GC_TX_OFFS should be set to 2.
     */

    /*
    7.3.4 TX Output Power
    The maximum output power of the transmitter is typically 5 dBm in normal mode and 10 dBm in boost mode.
    The TX output power can be set via register bits TX_PWR (register 0x05, PHY_TX_PWR).
    The output power of the transmitter can be controlled down to -11 dBm dB with 1 dB resolution.
    To meet the spectral requirements of the European and Chinese bands, it is necessary
    to limit the TX power by appropriate setting of TX_PWR, GC_PA (register 0x05, PHY_TX_PWR),
    and GC_TX_OFFS (register 0x16, TX_CTRL_0). See Table 7-15 and Table 7-16 for recommended values.
    */

    printf("Step 5> Set TX output power\n");
    value = phyReadRegister(PHY_TX_PWR_REG);
    printf("before PHY_TX_PWR_REG: 0x%02X\n", value);
    value = phyReadRegister(RF_CTRL_0_REG);
    printf("before RF_CTRL_0_REG: 0x%02X\n", value);

    /*
    915 MHz North American Band
    10 dBm 0xe1
    5  dBm 0x84
    0  dBm 0x24
    */
    phyWriteRegister(PHY_TX_PWR_REG, 0x24);

    /* For CW mode, GC_TX_OFFS should be set to 2 */
    value_write = (value & 0xFC) | 0x2;
    phyWriteRegister(RF_CTRL_0_REG, value_write);
    
    value = phyReadRegister(PHY_TX_PWR_REG);
    printf("after PHY_TX_PWR_REG: 0x%02X\n", value);
    value = phyReadRegister(RF_CTRL_0_REG);
    printf("after RF_CTRL_0_REG: 0x%02X\n", value);

    /*
     * Step 6. Register access 0x01 R 0x08 Verify TRX_OFF state
     */
    printf("Step 6> Verify TRX_OFF state\n");
    while(TRX_STATUS_TRX_OFF != (phyReadRegister(TRX_STATUS_REG) & TRX_STATUS_TRX_STATUS_MASK))
    {
        printf("Wait TRX_OFF state ...\n");
        value = phyReadRegister(TRX_STATUS_REG);
        printf("TRX_STATUS_REG: 0x%02X\n", value);
        Delay(200);
    }
    printf("Verify TRX_OFF state done ...\n");
    value = phyReadRegister(TRX_STATUS_REG);
    printf("TRX_STATUS_REG: 0x%02X\n", value);

    /*
     * Step 7. Register access 0x36 W 0x0F Enable Continuous Transmission Test Mode . step # 1
     */
    printf("Step 7> Enable Continuous Transmission Test Mode . step # 1\n");
    value = phyReadRegister(0x36);
    printf("before 0x36 Reg: 0x%02X\n", value);
    phyWriteRegister(0x36, 0x0F);
    value = phyReadRegister(0x36);
    printf("after 0x36 Reg: 0x%02X\n", value);

    /*
     * Step 8. Register access 0x0C W Select PRBS mode with modulation scheme or CW mode with carrier position:
     * 0x00 - PRBS mode, BPSK-20
     * 0x04 - PRBS mode, BPSK-40
     * 0x08 - PRBS mode, OQPSK-SIN-RC-100
     * 0x0C - PRBS mode, OQPSK-SIN-250
     * 0x1C - PRBS mode, OQPSK-RC-250
     * 0x0A - CW mode, CW at Fc  0.1 MHz
     * 0x0E - CW mode, CW at Fc  0.25 MHz
     */
    printf("Step 8> Select mode - CW mode\n");
    value = phyReadRegister(TRX_CTRL_2_REG);
    printf("before TRX_CTRL_2_REG: 0x%02X\n", value);
    phyWriteRegister(TRX_CTRL_2_REG, 0x0A);
    value = phyReadRegister(TRX_CTRL_2_REG);
    printf("after TRX_CTRL_2_REG: 0x%02X\n", value);

    /*
     * Step 9. Frame Buffer write access
     * {PHR, PSDU} PRBS mode: Write PHR value (0x01 . 0x7F) followed by PSDU data.
                PHR determines how many bytes of the PSDU data are repeated continuously.
     * {0x01, 0x00} CW mode, CW at Fc - 0.1 MHz
     * {0x01, 0xFF} CW mode, CW at Fc + 0.1 MHz
     * {0x01, 0x00} CW mode, CW at Fc - 0.25 MHz
     * {0x01, 0xFF} CW mode, CW at Fc + 0.25 MHz
     */
    printf("Step 9> Frame Buffer write access\n");
    {
    	uint8_t txcwdata[128];

		txcwdata[0] = 1; /* length */
		txcwdata[1] = 0;

		/* Frame buffer access */
        AT86RF212_SPI_BEGIN();
        HAL_PhySpiWriteByte(RF_CMD_FRAME_W);
        HAL_PhySpiWriteByte(2 + 2 /* crc */);
        HAL_PhySpiWriteByte(txcwdata[0]);
        HAL_PhySpiWriteByte(txcwdata[1]);
        AT86RF212_SPI_END();
    }

    /*
     * Step 10. Register access 0x1C W 0x54 Enable Continuous Transmission Test Mode . step # 2
     */
    phyWriteRegister(PART_NUM_REG, 0x54);
    printf("Step 10> Continuous TX step#2 PART_NUM_REG set 0x54 done\n");

    /*
     * Step 11. Register access 0x1C W 0x46 Enable Continuous Transmission Test Mode . step # 3
     */
    phyWriteRegister(PART_NUM_REG, 0x46);
    printf("Step 11> Continuous TX step#3 PART_NUM_REG set 0x46 done\n");

    /*
     * Step 12. Register access 0x02 W 0x09 Enable PLL_ON state
     */
    printf("Step 12> Enable PLL_ON state\n");
    value = phyReadRegister(TRX_STATE_REG);
    printf("before TRX_STATE_REG: 0x%02X\n", value);
    phyWriteRegister(TRX_STATE_REG, TRX_CMD_PLL_ON);
    value = phyReadRegister(TRX_STATE_REG);
    printf("after TRX_STATE_REG: 0x%02X\n", value);
    Delay(300);

    /*
     * Step 13. Interrupt event 0x0F R 0x01 Wait for IRQ_0 (PLL_LOCK)
     */
    printf("Step 13> Wait for IRQ_0 (PLL_LOCK)\n");
    value = phyReadRegister(IRQ_STATUS_REG);

    if(PLL_LOCK_MASK == value)
    {
        printf("PLL_ON Success! IRQ_STATUS_REG: 0x%02X\n", value);
    }
    else
    {
        printf("[[## PLL_ON FAIL !!##]] IRQ_STATUS_REG: 0x%02X\n", value);
        return;
    }

    /*
     * Step 14. Register access 0x02 W 0x02 Initiate transmission, enter BUSY_TX state
     */
    printf("Step 14> Initiate transmission, enter BUSY_TX state\n");
    phyWriteRegister(TRX_STATE_REG, TRX_CMD_TX_START);

    /*
     * Step 15. Measurement Perform measurement
     */
    printf("Step 15> Measurement Perform measurement\n");

    while(1)
    {
        Delay(500);
        value = phyReadRegister(TRX_STATUS_REG);
        printf("TRX_STATUS_REG: 0x%02X\n", value);
    }
}

void test_CW_signal_TX_Stop(void)
{
    /*
     * Step 16. Register access 0x1C W 0x00 Disable Continuous Transmission Test Mode
     */
    phyWriteRegister(PART_NUM_REG, 0x00);
    printf("Step 16> Disable Continuous TX Test, PART_NUM_REG set 0x00 done\n");

    /*
     * Step 17. Reset AT86RF212
     */
    AT86RF212_RESET();
    printf("Step 17> Reset done\n");
    Delay(100);
}

void AT86RF212_Test(SPI_TypeDef* SPIx_p)
{
    uint8_t ch = 0;
    GPIO_InitTypeDef GPIO_InitStructure;

    DEBUG_MSG_FUNC_START;

    SPI_Configuration();

    g_at86rf212_spi_num_p = SPIx_p;

    if(SPI1 == SPIx_p)
    {
        g_at86rf212_gpio_ncs         = GPIO_SPI_1;
        g_at86rf212_gpio_pin_ncs     = GPIO_SPI_1_NCS_PIN;
        g_at86rf212_gpio_pin_reset   = GPIO_SPI_IO_PIN_7;

        g_at86rf212_gpio_pin_0       = GPIO_SPI_IO_PIN_0;
        g_at86rf212_gpio_pin_1       = GPIO_SPI_IO_PIN_1;
        g_at86rf212_gpio_pin_2       = GPIO_SPI_IO_PIN_2;
        g_at86rf212_gpio_pin_3       = GPIO_SPI_IO_PIN_3;
        g_at86rf212_gpio_pin_4       = GPIO_SPI_IO_PIN_4;
        g_at86rf212_gpio_pin_5       = GPIO_SPI_IO_PIN_5;

        g_at86rf212_gpio_pin_0_irq        = EXTI0_IRQn;
        g_at86rf212_gpio_pin_0_pin_source = GPIO_PinSource0;
        g_at86rf212_gpio_pin_0_exti_line  = EXTI_Line0;
    }
    else
    {
        g_at86rf212_gpio_ncs         = GPIO_SPI_2;
        g_at86rf212_gpio_pin_ncs     = GPIO_SPI_2_NCS_PIN;
        g_at86rf212_gpio_pin_reset   = GPIO_SPI_IO_PIN_15;

        g_at86rf212_gpio_pin_0       = GPIO_SPI_IO_PIN_8;
        g_at86rf212_gpio_pin_1       = GPIO_SPI_IO_PIN_9;
        g_at86rf212_gpio_pin_2       = GPIO_SPI_IO_PIN_10;
        g_at86rf212_gpio_pin_3       = GPIO_SPI_IO_PIN_11;
        g_at86rf212_gpio_pin_4       = GPIO_SPI_IO_PIN_12;
        g_at86rf212_gpio_pin_5       = GPIO_SPI_IO_PIN_13;

        g_at86rf212_gpio_pin_0_irq        = EXTI9_5_IRQn;
        g_at86rf212_gpio_pin_0_pin_source = GPIO_PinSource8;
        g_at86rf212_gpio_pin_0_exti_line  = EXTI_Line8;
    }

    /* Configure gpio as input for interrupt */
    GPIO_InitStructure.GPIO_Pin   = g_at86rf212_gpio_pin_0;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIO_SPI_IO, &GPIO_InitStructure);

    /* Configure pins: RESETn ---------------------------------*/
    GPIO_InitStructure.GPIO_Pin   = g_at86rf212_gpio_pin_reset;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_PP;
    GPIO_Init(GPIO_SPI_IO, &GPIO_InitStructure);

    // IO7 pin sould be Low
    AT86RF212_RESET();

    printf("\n\n---------------------\n");
    printf("Press menu key\n");
    printf("---------------------\n");
    printf("1> Test: TX\n");
    printf("2> Test: RX\n");
    printf("---------------------\n");
    printf("9> Read ID\n");
    printf("---------------------\n");
    printf("a> CW TX Start\n");
    printf("b> CW TX Stop\n");
    printf("---------------------\n");
    printf("x> quit\n\n");

#if 1
#if 0
    ch = USART_GetCharacter(USART1);
#else
    ch = UartSerialGetChar_Wait();
#endif
#else
    ch = '1';
#endif

    printf("%c is selected\n", ch);

    switch((char)ch)
    {
    case '1':
        printf("Test Mode is TX\n");
        AT86RF212_Test_Peer2Peer_main(AT86RF212_TEST_MODE_TX);
        break;

    case '2':
        printf("Test Mode is RX\n");
        AT86RF212_Test_Peer2Peer_main(AT86RF212_TEST_MODE_RX);
        break;

    case '9':
        printf("PART_NUM_REG: 0x%02X, VERSION_NUM_REG: 0x%02X\n",
            phyReadRegister(PART_NUM_REG), phyReadRegister(VERSION_NUM_REG));
        return;

    case 'a':
        printf("Continuous Wave TX Start\n");
        test_CW_signal_TX_Start();
        return;

    case 'b':
        printf("Continuous Wave TX Stop\n");
        test_CW_signal_TX_Stop();
        return;

    case 'x': case 'X':
    default:
        return;
    }
}
