/*
 * zigbee_test.c
 */

/* Includes ------------------------------------------------------------------*/
#include "hw_config.h"
#include "cc1120_mango_porting.h"
#include "cc1120_test.h"
#include "string.h"
#include "debug.h"
#include "cc112x_spi.h"
#include "stdlib.h"
#include "cc112x_easy_link_reg_config.h"

/*******************************************************************************
* DEFINES
*/
#define ISR_ACTION_REQUIRED 1

/******************************************************************************
* LOCAL VARIABLES
*/
uint8  packetSemaphore;

/***********************************************************************************
 * CONSTANTS
 */

GPIO_TypeDef *  g_cc1120_gpio_ncs;
SPI_TypeDef *   g_cc1120_spi_num_p;

uint16_t  g_cc1120_gpio_pin_reset, g_cc1120_gpio_pin_ncs;
uint16_t  g_cc1120_gpio_pin_0, g_cc1120_gpio_pin_1,
    g_cc1120_gpio_pin_2, g_cc1120_gpio_pin_3, g_cc1120_gpio_pin_4, g_cc1120_gpio_pin_5;

uint8_t   g_cc1120_gpio_pin_5_pin_source, g_cc1120_gpio_pin_5_irq;
uint32_t  g_cc1120_gpio_pin_5_exti_line = EXTI_Line5;

uint8_t   g_cc1120_gpio_pin_6_pin_source, g_cc1120_gpio_pin_6_irq;
uint32_t  g_cc1120_gpio_pin_6_exti_line = EXTI_Line6;

void cc112xRunTX(void);
void cc112xRunRX(void);

/*******************************************************************************
* @fn          CC1120_radioRxTxISR
* @brief       ISR for packet handling. Sets packet semaphore 
* @param       none
* @return      none
*/
void CC1120_radioRxTxISR(void)
{
    // Set packet semaphore
    packetSemaphore = ISR_ACTION_REQUIRED;  
}

static void SPI_Configuration(void)
{
    SPI_InitTypeDef  SPI_InitStructure;

    /* SPI Config */
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_Mode      = SPI_Mode_Master;
    SPI_InitStructure.SPI_DataSize  = SPI_DataSize_8b;

    SPI_InitStructure.SPI_CPOL    = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA    = SPI_CPHA_1Edge;

    SPI_InitStructure.SPI_NSS               = SPI_NSS_Soft;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_16;
    SPI_InitStructure.SPI_FirstBit          = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial     = 7;

    /* Enable SPI1 */
    SPI_Init(SPI1, &SPI_InitStructure);
    SPI_Cmd(SPI1, ENABLE);

    /* Enable SPI2 */
    SPI_Init(SPI2, &SPI_InitStructure);
    SPI_Cmd(SPI2, ENABLE);
}

static const registerSetting_t preferredSettings_Modulation[] = {
    {CC112X_IOCFG3,         0xB0},  // IOCFG3              GPIO3 IO Pin Configuration
    {CC112X_IOCFG2,         0x08},  // IOCFG2              GPIO2 IO Pin Configuration
    {CC112X_IOCFG1,         0xB0},  // IOCFG1              GPIO1 IO Pin Configuration
    {CC112X_IOCFG0,         0x09},  // IOCFG0              GPIO0 IO Pin Configuration
    {CC112X_SYNC_CFG1,      0x0B},  // SYNC_CFG1           Sync Word Detection Configuration Reg. 1
    {CC112X_DCFILT_CFG,     0x1C},  // DCFILT_CFG          Digital DC Removal Configuration
    {CC112X_PREAMBLE_CFG1,  0x00},  // PREAMBLE_CFG1       Preamble Length Configuration Reg. 1
    {CC112X_IQIC,           0xC6},  // IQIC                Digital Image Channel Compensation Configuration
    {CC112X_CHAN_BW,        0x08},  // CHAN_BW             Channel Filter Configuration
    {CC112X_MDMCFG1,        0x06},  // MDMCFG1             General Modem Parameter Configuration Reg. 1
    {CC112X_MDMCFG0,        0x05},  // MDMCFG0             General Modem Parameter Configuration Reg. 0
    {CC112X_AGC_REF,        0x20},  // AGC_REF             AGC Reference Level Configuration
    {CC112X_AGC_CS_THR,     0x19},  // AGC_CS_THR          Carrier Sense Threshold Configuration
    {CC112X_AGC_CFG1,       0xA9},  // AGC_CFG1            Automatic Gain Control Configuration Reg. 1
    {CC112X_AGC_CFG0,       0xCF},  // AGC_CFG0            Automatic Gain Control Configuration Reg. 0
    {CC112X_FIFO_CFG,       0x00},  // FIFO_CFG            FIFO Configuration
    {CC112X_SETTLING_CFG,   0x03},  // SETTLING_CFG        Frequency Synthesizer Calibration and Settling Con..
    {CC112X_FS_CFG,         0x12},  // FS_CFG              Frequency Synthesizer Configuration
    {CC112X_PKT_CFG2,       0x05},  // PKT_CFG2            Packet Configuration Reg. 2
    {CC112X_PKT_CFG1,       0x00},  // PKT_CFG1            Packet Configuration Reg. 1
    {CC112X_PKT_CFG0,       0x20},  // PKT_CFG0            Packet Configuration Reg. 0
    {CC112X_PA_CFG2,        0x3F},  // PA_CFG2             Power Amplifier Configuration Reg. 2
    {CC112X_IF_MIX_CFG,     0x00},  // IF_MIX_CFG          IF Mix Configuration
    {CC112X_FREQOFF_CFG,    0x22},  // FREQOFF_CFG         Frequency Offset Correction Configuration
    {CC112X_FREQ2,          0x6C},  // FREQ2               Frequency Configuration [23:16]
    {CC112X_FREQ1,          0x80},  // FREQ1               Frequency Configuration [15:8]
    {CC112X_FS_DIG1,        0x00},  // FS_DIG1             Frequency Synthesizer Digital Reg. 1
    {CC112X_FS_DIG0,        0x5F},  // FS_DIG0             Frequency Synthesizer Digital Reg. 0
    {CC112X_FS_CAL1,        0x40},  // FS_CAL1             Frequency Synthesizer Calibration Reg. 1
    {CC112X_FS_CAL0,        0x0E},  // FS_CAL0             Frequency Synthesizer Calibration Reg. 0
    {CC112X_FS_DIVTWO,      0x03},  // FS_DIVTWO           Frequency Synthesizer Divide by 2
    {CC112X_FS_DSM0,        0x33},  // FS_DSM0             FS Digital Synthesizer Module Configuration Reg. 0
    {CC112X_FS_DVC0,        0x17},  // FS_DVC0             Frequency Synthesizer Divider Chain Configuration ..
    {CC112X_FS_PFD,         0x50},  // FS_PFD              Frequency Synthesizer Phase Frequency Detector Con..
    {CC112X_FS_PRE,         0x6E},  // FS_PRE              Frequency Synthesizer Prescaler Configuration
    {CC112X_FS_REG_DIV_CML, 0x14},  // FS_REG_DIV_CML      Frequency Synthesizer Divider Regulator Configurat..
    {CC112X_FS_SPARE,       0xAC},  // FS_SPARE            Frequency Synthesizer Spare
    {CC112X_FS_VCO0,        0xB4},  // FS_VCO0             FS Voltage Controlled Oscillator Configuration Reg..
    {CC112X_XOSC5,          0x0E},  // XOSC5               Crystal Oscillator Configuration Reg. 5
    {CC112X_XOSC1,          0x03},  // XOSC1               Crystal Oscillator Configuration Reg. 1
    {CC112X_PARTNUMBER,     0x48},  // PARTNUMBER          Part Number
    {CC112X_PARTVERSION,    0x21},  // PARTVERSION         Part Revision
    {CC112X_SERIAL_STATUS,  0x08},  // SERIAL_STATUS       Serial Status
    {CC112X_MODEM_STATUS1,  0x10},  // MODEM_STATUS1       Modem Status Reg. 1
};

static const registerSetting_t preferredSettings_UnModulation[] = {
    {CC112X_IOCFG3,         0xB0},  // IOCFG3              GPIO3 IO Pin Configuration
    {CC112X_IOCFG2,         0x08},  // IOCFG2              GPIO2 IO Pin Configuration
    {CC112X_IOCFG1,         0xB0},  // IOCFG1              GPIO1 IO Pin Configuration
    {CC112X_IOCFG0,         0x09},  // IOCFG0              GPIO0 IO Pin Configuration
    {CC112X_SYNC_CFG1,      0x0B},  // SYNC_CFG1           Sync Word Detection Configuration Reg. 1
    {CC112X_DCFILT_CFG,     0x1C},  // DCFILT_CFG          Digital DC Removal Configuration
    {CC112X_PREAMBLE_CFG1,  0x00},  // PREAMBLE_CFG1       Preamble Length Configuration Reg. 1
    {CC112X_IQIC,           0xC6},  // IQIC                Digital Image Channel Compensation Configuration
    {CC112X_CHAN_BW,        0x08},  // CHAN_BW             Channel Filter Configuration
    {CC112X_MDMCFG1,        0x06},  // MDMCFG1             General Modem Parameter Configuration Reg. 1
    {CC112X_MDMCFG0,        0x05},  // MDMCFG0             General Modem Parameter Configuration Reg. 0
    {CC112X_AGC_REF,        0x20},  // AGC_REF             AGC Reference Level Configuration
    {CC112X_AGC_CS_THR,     0x19},  // AGC_CS_THR          Carrier Sense Threshold Configuration
    {CC112X_AGC_CFG1,       0xA9},  // AGC_CFG1            Automatic Gain Control Configuration Reg. 1
    {CC112X_AGC_CFG0,       0xCF},  // AGC_CFG0            Automatic Gain Control Configuration Reg. 0
    {CC112X_FIFO_CFG,       0x00},  // FIFO_CFG            FIFO Configuration
    {CC112X_SETTLING_CFG,   0x03},  // SETTLING_CFG        Frequency Synthesizer Calibration and Settling Con..
    {CC112X_FS_CFG,         0x12},  // FS_CFG              Frequency Synthesizer Configuration
    {CC112X_PKT_CFG2,       0x05},  // PKT_CFG2            Packet Configuration Reg. 2
    {CC112X_PKT_CFG1,       0x00},  // PKT_CFG1            Packet Configuration Reg. 1
    {CC112X_PKT_CFG0,       0x20},  // PKT_CFG0            Packet Configuration Reg. 0
    {CC112X_PA_CFG2,        0x3F},  // PA_CFG2             Power Amplifier Configuration Reg. 2
    {CC112X_IF_MIX_CFG,     0x00},  // IF_MIX_CFG          IF Mix Configuration
    {CC112X_FREQOFF_CFG,    0x22},  // FREQOFF_CFG         Frequency Offset Correction Configuration

    {CC112X_CFM_DATA_CFG,   0x01},  // CFM_DATA_CFG        Custom frequency modulation enable

    {CC112X_FREQ2,          0x6C},  // FREQ2               Frequency Configuration [23:16]
    {CC112X_FREQ1,          0x80},  // FREQ1               Frequency Configuration [15:8]
    {CC112X_FS_DIG1,        0x00},  // FS_DIG1             Frequency Synthesizer Digital Reg. 1
    {CC112X_FS_DIG0,        0x5F},  // FS_DIG0             Frequency Synthesizer Digital Reg. 0
    {CC112X_FS_CAL1,        0x40},  // FS_CAL1             Frequency Synthesizer Calibration Reg. 1
    {CC112X_FS_CAL0,        0x0E},  // FS_CAL0             Frequency Synthesizer Calibration Reg. 0
    {CC112X_FS_DIVTWO,      0x03},  // FS_DIVTWO           Frequency Synthesizer Divide by 2
    {CC112X_FS_DSM0,        0x33},  // FS_DSM0             FS Digital Synthesizer Module Configuration Reg. 0
    {CC112X_FS_DVC0,        0x17},  // FS_DVC0             Frequency Synthesizer Divider Chain Configuration ..
    {CC112X_FS_PFD,         0x50},  // FS_PFD              Frequency Synthesizer Phase Frequency Detector Con..
    {CC112X_FS_PRE,         0x6E},  // FS_PRE              Frequency Synthesizer Prescaler Configuration
    {CC112X_FS_REG_DIV_CML, 0x14},  // FS_REG_DIV_CML      Frequency Synthesizer Divider Regulator Configurat..
    {CC112X_FS_SPARE,       0xAC},  // FS_SPARE            Frequency Synthesizer Spare
    {CC112X_FS_VCO0,        0xB4},  // FS_VCO0             FS Voltage Controlled Oscillator Configuration Reg..
    {CC112X_XOSC5,          0x0E},  // XOSC5               Crystal Oscillator Configuration Reg. 5
    {CC112X_XOSC1,          0x03},  // XOSC1               Crystal Oscillator Configuration Reg. 1
    {CC112X_PARTNUMBER,     0x48},  // PARTNUMBER          Part Number
    {CC112X_PARTVERSION,    0x21},  // PARTVERSION         Part Revision
    {CC112X_SERIAL_STATUS,  0x08},  // SERIAL_STATUS       Serial Status
    {CC112X_MODEM_STATUS1,  0x10},  // MODEM_STATUS1       Modem Status Reg. 1
};

static void test_set_reg_Modulation(void)
{
    uint8 writeByte;

    // Reset radio
    trxSpiCmdStrobe(CC112X_SRES);

    // Write registers to radio
    for(uint16 i = 0; i < (sizeof  preferredSettings_Modulation/sizeof(registerSetting_t)); i++)
    {
        writeByte =  preferredSettings_Modulation[i].data;
        cc112xSpiWriteReg( preferredSettings_Modulation[i].addr, &writeByte, 1);
    }
}

static void test_set_reg_UnModulation(void)
{
    uint8 writeByte;

    // Reset radio
    trxSpiCmdStrobe(CC112X_SRES);

    // Write registers to radio
    for(uint16 i = 0; i < (sizeof  preferredSettings_UnModulation/sizeof(registerSetting_t)); i++)
    {
        writeByte =  preferredSettings_UnModulation[i].data;
        cc112xSpiWriteReg( preferredSettings_UnModulation[i].addr, &writeByte, 1);
    }
}

void CC1120_Test(SPI_TypeDef* SPIx_p)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    u8 ch;
    uint8 readData;

    DEBUG_MSG_FUNC_START;

    SPI_Configuration();

    g_cc1120_spi_num_p = SPIx_p;

    if(SPI1 == SPIx_p)
    {
        g_cc1120_gpio_ncs         = GPIO_SPI_1;
        g_cc1120_gpio_pin_ncs     = GPIO_SPI_1_NCS_PIN;
        g_cc1120_gpio_pin_reset   = GPIO_SPI_IO_PIN_7;

        g_cc1120_gpio_pin_0       = GPIO_SPI_IO_PIN_0;
        g_cc1120_gpio_pin_1       = GPIO_SPI_IO_PIN_1;
        g_cc1120_gpio_pin_2       = GPIO_SPI_IO_PIN_2;
        g_cc1120_gpio_pin_3       = GPIO_SPI_IO_PIN_3;
        g_cc1120_gpio_pin_4       = GPIO_SPI_IO_PIN_4;
        g_cc1120_gpio_pin_5       = GPIO_SPI_IO_PIN_5;

        g_cc1120_gpio_pin_5_irq        = EXTI9_5_IRQn;
        g_cc1120_gpio_pin_5_pin_source = GPIO_PinSource5;
        g_cc1120_gpio_pin_5_exti_line  = EXTI_Line5;

        g_cc1120_gpio_pin_6_irq        = EXTI9_5_IRQn;
        g_cc1120_gpio_pin_6_pin_source = GPIO_PinSource6;
        g_cc1120_gpio_pin_6_exti_line  = EXTI_Line6;
    }
    else
    {
        g_cc1120_gpio_ncs         = GPIO_SPI_2;
        g_cc1120_gpio_pin_ncs     = GPIO_SPI_2_NCS_PIN;
        g_cc1120_gpio_pin_reset   = GPIO_SPI_IO_PIN_15;

        g_cc1120_gpio_pin_0       = GPIO_SPI_IO_PIN_8;
        g_cc1120_gpio_pin_1       = GPIO_SPI_IO_PIN_9;
        g_cc1120_gpio_pin_2       = GPIO_SPI_IO_PIN_10;
        g_cc1120_gpio_pin_3       = GPIO_SPI_IO_PIN_11;
        g_cc1120_gpio_pin_4       = GPIO_SPI_IO_PIN_12;
        g_cc1120_gpio_pin_5       = GPIO_SPI_IO_PIN_13;

        g_cc1120_gpio_pin_5_irq        = EXTI15_10_IRQn;
        g_cc1120_gpio_pin_5_pin_source = GPIO_PinSource13;
        g_cc1120_gpio_pin_5_exti_line  = EXTI_Line13;

        g_cc1120_gpio_pin_6_irq        = EXTI15_10_IRQn;
        g_cc1120_gpio_pin_6_pin_source = GPIO_PinSource14;
        g_cc1120_gpio_pin_6_exti_line  = EXTI_Line14;
    }

    CC1120_GPIO_0_Interrupt_Setting(ENABLE);
//    CC1120_GPIO_2_Interrupt_Setting(ENABLE);

    /* Configure gpio as input for interrupt */
    GPIO_InitStructure.GPIO_Pin   = g_cc1120_gpio_pin_5;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIO_SPI_IO, &GPIO_InitStructure);

    /* Configure pins: RESETn ---------------------------------*/
    GPIO_InitStructure.GPIO_Pin   = g_cc1120_gpio_pin_reset;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_PP;
    GPIO_Init(GPIO_SPI_IO, &GPIO_InitStructure);

    // IO7 pin sould be Low
    GPIO_SetBits(GPIO_SPI_IO, g_cc1120_gpio_pin_reset);
    Delay(100);
    GPIO_ResetBits(GPIO_SPI_IO, g_cc1120_gpio_pin_reset);
    Delay(100);

    // Write radio registers
//    registerConfig();

    Delay(2);

    // Read data
    cc112xSpiReadReg(CC112X_PARTNUMBER, &readData, 1);
    printf("CC1120 part num: 0x%02X\n", readData);

    cc112xSpiReadReg(CC112X_PARTVERSION, &readData, 1);
    printf("CC1120 part ver: 0x%02X\n", readData);

    while(1)
    {
        printf("\n\n------------------------\n");
        printf("(CC1120 Test) Press menu key\n");
        printf("------------------------\n");
        printf("1> Run Tx\n");
        printf("2> Run Rx\n");
        printf("---------------------\n");
        printf("a> Set reg Modulation\n");
        printf("b> Set reg UnModulation\n");
        printf("------------------------\n");
        printf("x> quit\n\n");

#if 1
#if 0
        ch = USART_GetCharacter(USART1);
#else
        ch = UartSerialGetChar_Wait();
#endif
#else
        ch = '4';
#endif

        printf("%c is selected\n", ch);

        switch((char)ch)
        {
        case '1':
            // Enter runTX, never coming back
            cc112xRunTX();
            break;

        case '2':
            // Enter runRX, never coming back
            cc112xRunRX();
            break;

        case 'a':
            test_set_reg_Modulation();

            // Strobe TX to send packet
            trxSpiCmdStrobe(CC112X_STX);
            break;

        case 'b':
            test_set_reg_UnModulation();

            // Strobe TX to send packet
            trxSpiCmdStrobe(CC112X_STX);
            break;
        }

        if('x' == (char)ch)
        {
            break;
        }
    }
}

/******************************************************************************
 * @fn          manualCalibration
 *
 * @brief       calibrates radio according to CC112x errata
 *                
 * @param       none
 *
 * @return      none
 */
#if 0

#define VCDAC_START_OFFSET 2
#define FS_VCO2_INDEX 0
#define FS_VCO4_INDEX 1
#define FS_CHP_INDEX 2

static void manualCalibration(void)
{
  uint8 original_fs_cal2;
  uint8 calResults_for_vcdac_start_high[3];
  uint8 calResults_for_vcdac_start_mid[3];
  uint8 marcstate;
  uint8 writeByte;
    
  // 1) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
  writeByte = 0x00;
  cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    
  // 2) Start with high VCDAC (original VCDAC_START + 2):
  cc112xSpiReadReg(CC112X_FS_CAL2, &original_fs_cal2, 1);
  writeByte = original_fs_cal2 + VCDAC_START_OFFSET;
  cc112xSpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);
    
  // 3) Calibrate and wait for calibration to be done (radio back in IDLE state)
  trxSpiCmdStrobe(CC112X_SCAL);
    
  do {
    cc112xSpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
  } while (marcstate != 0x41);
    
  // 4) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with high VCDAC_START value
  cc112xSpiReadReg(CC112X_FS_VCO2, &calResults_for_vcdac_start_high[FS_VCO2_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_VCO4, &calResults_for_vcdac_start_high[FS_VCO4_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_CHP, &calResults_for_vcdac_start_high[FS_CHP_INDEX], 1);
    
  // 5) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
  writeByte = 0x00;
  cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    
  // 6) Continue with mid VCDAC (original VCDAC_START):
  writeByte = original_fs_cal2;
  cc112xSpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);
    
  // 7) Calibrate and wait for calibration to be done (radio back in IDLE state)
  trxSpiCmdStrobe(CC112X_SCAL);
    
  do {
    cc112xSpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
  } while (marcstate != 0x41);

  // 8) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with mid VCDAC_START value
  cc112xSpiReadReg(CC112X_FS_VCO2, &calResults_for_vcdac_start_mid[FS_VCO2_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_VCO4, &calResults_for_vcdac_start_mid[FS_VCO4_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_CHP, &calResults_for_vcdac_start_mid[FS_CHP_INDEX], 1);
    
  // 9) Write back highest FS_VCO2 and corresponding FS_VCO and FS_CHP result
  if (calResults_for_vcdac_start_high[FS_VCO2_INDEX] > calResults_for_vcdac_start_mid[FS_VCO2_INDEX]) {
    writeByte = calResults_for_vcdac_start_high[FS_VCO2_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_high[FS_VCO4_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_high[FS_CHP_INDEX];
    cc112xSpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
  } else {
    writeByte = calResults_for_vcdac_start_mid[FS_VCO2_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_mid[FS_VCO4_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_mid[FS_CHP_INDEX];
    cc112xSpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
  }
}
#endif

