/*
 * enc28j60_test.c
 */

/* Includes ------------------------------------------------------------------*/
#include "enc28j60.h"

const unsigned char enc28j60_MAC[6] = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06};

GPIO_TypeDef *  g_enc28j60_gpio_ncs;
SPI_TypeDef *   g_enc28j60_spi_num_p;

uint16_t  g_enc28j60_gpio_pin_reset, g_enc28j60_gpio_pin_ncs;
uint16_t  g_enc28j60_gpio_pin_0, g_enc28j60_gpio_pin_1,
    g_enc28j60_gpio_pin_2, g_enc28j60_gpio_pin_3, g_enc28j60_gpio_pin_4, g_enc28j60_gpio_pin_5;

extern int simple_server(void);

static void SPI_Configuration(void)
{
    SPI_InitTypeDef  SPI_InitStructure;

    /* SPI Config */
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_Mode      = SPI_Mode_Master;
    SPI_InitStructure.SPI_DataSize  = SPI_DataSize_8b;

    SPI_InitStructure.SPI_CPOL    = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA    = SPI_CPHA_1Edge;

    SPI_InitStructure.SPI_NSS               = SPI_NSS_Soft;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_16;
    SPI_InitStructure.SPI_FirstBit          = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial     = 7;

    /* Enable SPI1 */
    SPI_Init(SPI1, &SPI_InitStructure);
    SPI_Cmd(SPI1, ENABLE);

    /* Enable SPI2 */
    SPI_Init(SPI2, &SPI_InitStructure);
    SPI_Cmd(SPI2, ENABLE);
}

void ENC28J60_Test(SPI_TypeDef* SPIx_p)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    int rev = 0;

    DEBUG_MSG_FUNC_START;

    SPI_Configuration();

    g_enc28j60_spi_num_p = SPIx_p;

    if(SPI1 == SPIx_p)
    {
        g_enc28j60_gpio_ncs         = GPIO_SPI_1;
        g_enc28j60_gpio_pin_ncs     = GPIO_SPI_1_NCS_PIN;
        g_enc28j60_gpio_pin_reset   = GPIO_SPI_IO_PIN_7;

        g_enc28j60_gpio_pin_0       = GPIO_SPI_IO_PIN_0;
        g_enc28j60_gpio_pin_1       = GPIO_SPI_IO_PIN_1;
        g_enc28j60_gpio_pin_2       = GPIO_SPI_IO_PIN_2;
        g_enc28j60_gpio_pin_3       = GPIO_SPI_IO_PIN_3;
        g_enc28j60_gpio_pin_4       = GPIO_SPI_IO_PIN_4;
        g_enc28j60_gpio_pin_5       = GPIO_SPI_IO_PIN_5;
    }
    else
    {
        g_enc28j60_gpio_ncs         = GPIO_SPI_2;
        g_enc28j60_gpio_pin_ncs     = GPIO_SPI_2_NCS_PIN;
        g_enc28j60_gpio_pin_reset   = GPIO_SPI_IO_PIN_15;

        g_enc28j60_gpio_pin_0       = GPIO_SPI_IO_PIN_8;
        g_enc28j60_gpio_pin_1       = GPIO_SPI_IO_PIN_9;
        g_enc28j60_gpio_pin_2       = GPIO_SPI_IO_PIN_10;
        g_enc28j60_gpio_pin_3       = GPIO_SPI_IO_PIN_11;
        g_enc28j60_gpio_pin_4       = GPIO_SPI_IO_PIN_12;
        g_enc28j60_gpio_pin_5       = GPIO_SPI_IO_PIN_13;
    }

    /* Configure pins: RESETn ---------------------------------*/
    GPIO_InitStructure.GPIO_Pin   = g_enc28j60_gpio_pin_reset;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_PP;
    GPIO_Init(GPIO_SPI_IO, &GPIO_InitStructure);

    // IO7 pin sould be Low
    GPIO_SetBits(GPIO_SPI_IO, g_enc28j60_gpio_pin_reset);
    Delay(100);
    GPIO_ResetBits(GPIO_SPI_IO, g_enc28j60_gpio_pin_reset);
    Delay(100);

//    SAMDK_Init();
    
    //__enable_irq();

    simple_server();

    printf("enc28j60_MAC: 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, \n",
        enc28j60_MAC[0], enc28j60_MAC[1], enc28j60_MAC[2], enc28j60_MAC[3], enc28j60_MAC[4], enc28j60_MAC[5]);

    enc28j60Init((unsigned char *)enc28j60_MAC);

    rev = enc28j60getrev();
    printf("rev: 0x%08X\n", rev);
}

